/*
 * C
 *
 * Copyright 2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/* Includes --------------------------------------------------------------------------*/
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <sni.h>
#include "LLNET_CONSTANTS.h"
#include "LLNET_ERRORS.h"
#include "NETIF_interface.h"
#include "NET_delegate_task.h"
#include "LLECOM_NETWORK_impl.h"


//#define LLECOM_NETWORK_ENABLE_TRACE

#ifdef LLECOM_NETWORK_ENABLE_TRACE
#define ASSERT_LLECOM_NETWORK(x) do{if(!(x)){printf("assert_llecom_network\n");}}while(0)
#else
#define ASSERT_LLECOM_NETWORK(x) ((void) 0)
#endif

#define IF_IP_STATIC		(0)
#define IF_IP_DYNAMIC		(1)


int32_t LLECOM_NETWORK_IMPL_disable(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	int32_t i_return = 0;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult);
	#endif

	if (NETIF_set_is_up_f((char*) (netifName + netifNameOffset), 0) != NETIF_NO_ERROR)
	{
		i_return = J_EUNKNOWN;
	}
	else
	{
		i_return = 0;
	}
	
	return i_return;
}

int32_t LLECOM_NETWORK_IMPL_enable(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	int32_t i_return;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);

	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s\n", __func__, (netifName + netifNameOffset));
	#endif

	if (NETIF_set_is_up_f((char*) (netifName + netifNameOffset), 1) != NETIF_NO_ERROR)
	{
		i_return = J_EUNKNOWN;
	}
	else
	{
		i_return = 0;
	}
	
	return i_return;
}

int32_t LLECOM_NETWORK_IMPL_isEnabled(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	unsigned char uc_is_enabled;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s\n", __func__, (netifName + netifNameOffset));
	#endif
	
	if (NETIF_get_is_up_f((char*) (netifName + netifNameOffset), &uc_is_enabled) == NETIF_NO_ERROR)
	{
		return (uc_is_enabled == 0) ? 0 : 1;
	}

	return 0;
}

int32_t LLECOM_NETWORK_IMPL_start(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	NET_event_t * pst_event;
	int32_t i_return = 0;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);	
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult);
	#endif

	// If not retry, this is the first execution so post event
	if (getResult != JTRUE)
	{
		if (NET_get_event(&pst_event) != NET_NO_ERROR)
		{
			i_return = (int32_t) J_ENOMEM;
		}
		else
		{
			pst_event->e_id_event = NET_START_EVENT;
			pst_event->u_ctx_event.st_start_ctx_t.pc_interface_name_in = (char*) (netifName + netifNameOffset);
			pst_event->u_ctx_event.st_start_ctx_t.i_interface_name_length_in = netifNameLength - netifNameOffset;

			if (NET_post_event(pst_event) != NET_NO_ERROR)
			{
				// An error occurred
				i_return = (int32_t) J_ENOMEM;
			}
			else
			{
				// Must return this code in case of on error to be recall with retry to true
				i_return = (int32_t) J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT;
			}
		}
	}
	else // Else it's second execution the thread has just been resumed by end of delegate process
	{
		// Retrieve event in pool according to java thread id
		if (NET_retrieve_event(&pst_event, SNI_getCurrentJavaThreadID()) != NET_NO_ERROR)
		{
			// An error occurred
			i_return = (int32_t) J_ENOMEM;
		}
		else
		{
			// Return value from context
			i_return = (int32_t)pst_event->u_ctx_event.st_start_ctx_t.s_error_out;

			// Release pool event
			if (NET_release_event(pst_event) != NET_NO_ERROR)
			{
				// An error occurred
				i_return = (int32_t) J_ENOMEM;
			}			
		}
	}
	
	return i_return;
}

int32_t LLECOM_NETWORK_IMPL_stop(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	NET_event_t * pst_event;
	int32_t i_return = 0;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult);
	#endif

	// If not retry, this is the first execution so post event
	if (getResult != JTRUE)
	{
		if (NET_get_event(&pst_event) != NET_NO_ERROR)
		{
			i_return = (int32_t) J_ENOMEM;
		}
		else
		{
			pst_event->e_id_event = NET_STOP_EVENT;
			pst_event->u_ctx_event.st_stop_ctx_t.pc_interface_name_in = (char*) (netifName + netifNameOffset);
			pst_event->u_ctx_event.st_stop_ctx_t.i_interface_name_length_in = netifNameLength - netifNameOffset;

			if (NET_post_event(pst_event) != NET_NO_ERROR)
			{
				// An error occurred
				i_return = (int32_t) J_ENOMEM;
			}
			else
			{
				// Must return this code in case of on error to be recall with retry to true
				i_return = (int32_t) J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT;
			}
		}
	}
	else // Else it's second execution the thread has just been resumed by end of delegate process
	{
		// Retrieve event in pool according to java thread id
		if (NET_retrieve_event(&pst_event, SNI_getCurrentJavaThreadID()) != NET_NO_ERROR)
		{
			// An error occurred
			i_return = (int32_t) J_ENOMEM;
		}
		else
		{
			// Return value from context
			i_return = (int32_t)pst_event->u_ctx_event.st_stop_ctx_t.s_error_out;

			// Release pool event
			if (NET_release_event(pst_event) != NET_NO_ERROR)
			{
				// An error occurred
				i_return = (int32_t) J_ENOMEM;
			}			
		}
	}
	
	return i_return;
}

int32_t LLECOM_NETWORK_IMPL_isStarted(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	unsigned char uc_is_started;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s\n", __func__, (netifName + netifNameOffset));
	#endif
	
	if (NETIF_is_started_f((char*) (netifName + netifNameOffset), &uc_is_started) == NETIF_NO_ERROR)
	{
		return (uc_is_started == 0) ? 0 : 1;
	}

	return 0;
}

/*- Functions to get the IP configuration of an interface -----------------------*/

int32_t LLECOM_NETWORK_IMPL_getDNSCount(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif
  
	// Wifly only supports one DNS server
	return 1;
}

int32_t LLECOM_NETWORK_IMPL_getDNS(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t* address, int32_t addressOffset, int32_t addressLength, int32_t index, int8_t getResult)
{
	int32_t i_return;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	ASSERT_LLECOM_NETWORK(address != NULL);
	ASSERT_LLECOM_NETWORK(addressOffset < addressLength);
	ASSERT_LLECOM_NETWORK(index == 0); // because there is only one DNS server for sure
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d %d\n", __func__, (netifName + netifNameOffset), index, getResult); 
	#endif

	if (NETIF_get_dns_f((char*) (netifName + netifNameOffset), (unsigned char *) (address + addressOffset)) == NETIF_NO_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}

  return i_return;
}

int32_t LLECOM_NETWORK_IMPL_getGateway(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t* address, int32_t addressOffset, int32_t addressLength, int8_t getResult)
{
	int32_t i_return;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	ASSERT_LLECOM_NETWORK(address != NULL);
	ASSERT_LLECOM_NETWORK(addressOffset < addressLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif

	if (NETIF_get_gateway_f((char*) (netifName + netifNameOffset), (unsigned char *) (address + addressOffset)) == NETIF_NO_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}

  return i_return;
}


int32_t LLECOM_NETWORK_IMPL_getIP(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t* address, int32_t addressOffset, int32_t addressLength, int8_t getResult)
{
	int32_t i_return;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	ASSERT_LLECOM_NETWORK(address != NULL);
	ASSERT_LLECOM_NETWORK(addressOffset < addressLength);

	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif

	if (NETIF_get_interface_ip_by_name_f((char*) (netifName + netifNameOffset), 0, (unsigned char*) (address + addressOffset)) != NETIF_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}
	
	return i_return;
}

int32_t LLECOM_NETWORK_IMPL_getNetmask(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t* address, int32_t addressOffset, int32_t addressLength, int8_t getResult)
{
	int32_t i_return;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	ASSERT_LLECOM_NETWORK(address != NULL);
	ASSERT_LLECOM_NETWORK(addressOffset < addressLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif

	if (NETIF_get_netmask_f((char*) (netifName + netifNameOffset), (unsigned char *) (address + addressOffset)) == NETIF_NO_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}

  return i_return;
}

int32_t LLECOM_NETWORK_IMPL_isDNSStatic(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif
	
	return 0;
}

int32_t LLECOM_NETWORK_IMPL_isStatic(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	unsigned char uc_is_static;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult);
	#endif
	
	if (NETIF_get_is_static_f((char*) (netifName + netifNameOffset), &uc_is_static) == NETIF_NO_ERROR)
	{
		return (uc_is_static == 0) ? IF_IP_DYNAMIC : IF_IP_STATIC;
	}

	return 0;
}

/*- Functions to set the IP configuration of an interface -----------------------*/

int32_t LLECOM_NETWORK_IMPL_endConfiguration(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	NET_event_t * pst_event;
	int32_t i_return = 0;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult);
	#endif

	// If not retry, this is the first execution so post event
	if (getResult != JTRUE)
	{
		if (NET_get_event(&pst_event) != NET_NO_ERROR)
		{
			i_return = (int32_t) J_ENOMEM;
		}
		else
		{
			pst_event->e_id_event = NET_APPLY_CONFIGURATION_EVENT;
			pst_event->u_ctx_event.st_apply_configuration_ctx_t.pc_interface_name_in = (char*) (netifName + netifNameOffset);
			pst_event->u_ctx_event.st_apply_configuration_ctx_t.i_interface_name_length_in = netifNameLength - netifNameOffset;

			if (NET_post_event(pst_event) != NET_NO_ERROR)
			{
				// An error occurred
				i_return = (int32_t) J_ENOMEM;
			}
			else
			{
				// Must return this code in case of on error to be recall with retry to true
				i_return = (int32_t) J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT;
			}
		}
	}
	else // Else it's second execution the thread has just been resumed by end of delegate process
	{
		// Retrieve event in pool according to java thread id
		if (NET_retrieve_event(&pst_event, SNI_getCurrentJavaThreadID()) != NET_NO_ERROR)
		{
			// An error occurred
			i_return = (int32_t) J_ENOMEM;
		}
		else
		{
			// Return value from context
			i_return = (int32_t)pst_event->u_ctx_event.st_apply_configuration_ctx_t.s_error_out;

			// Release pool event
			if (NET_release_event(pst_event) != NET_NO_ERROR)
			{
				// An error occurred
				i_return = (int32_t) J_ENOMEM;
			}			
		}
	}
	
	return i_return;
}

int32_t LLECOM_NETWORK_IMPL_startConfiguration(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t getResult)
{
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif
  
	return 0;
}

int32_t LLECOM_NETWORK_IMPL_setDNS(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t* address, int32_t addressOffset, int32_t addressLength, int32_t index, int8_t getResult)
{
	int32_t i_return;

	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	ASSERT_LLECOM_NETWORK(address != NULL);
	ASSERT_LLECOM_NETWORK(addressOffset < addressLength);
	ASSERT_LLECOM_NETWORK(index == 0); // because there is only one DNS server for sure

	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d %d\n", __func__, (netifName + netifNameOffset), index, getResult); 
	#endif

	if (NETIF_set_dns_f((char*) (netifName + netifNameOffset), (unsigned char *) (address + addressOffset)) == NETIF_NO_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}
		
  return i_return;
}

int32_t LLECOM_NETWORK_IMPL_setGateway(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t* address, int32_t addressOffset, int32_t addressLength, int8_t getResult)
{
	int32_t i_return;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	ASSERT_LLECOM_NETWORK(address != NULL);
	ASSERT_LLECOM_NETWORK(addressOffset < addressLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset),  getResult); 
	#endif

	if (NETIF_set_gateway_f((char*) (netifName + netifNameOffset), (unsigned char *) (address + addressOffset)) == NETIF_NO_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}
		
  return i_return;
}

int32_t LLECOM_NETWORK_IMPL_setIP(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t* address, int32_t addressOffset, int32_t addressLength, int8_t getResult)
{
	int32_t i_return;

	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	ASSERT_LLECOM_NETWORK(address != NULL);
	ASSERT_LLECOM_NETWORK(addressOffset < addressLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif
	
	if (NETIF_set_interface_ip_by_name_f((char*) (netifName + netifNameOffset), 0, (unsigned char*) (address + addressOffset)) != NETIF_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}
	
	return i_return;
}

int32_t LLECOM_NETWORK_IMPL_setNetmask(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t* address, int32_t addressOffset, int32_t addressLength, int8_t getResult)
{
	int32_t i_return;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	ASSERT_LLECOM_NETWORK(address != NULL);
	ASSERT_LLECOM_NETWORK(addressOffset < addressLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif

	if (NETIF_set_netmask_f((char*) (netifName + netifNameOffset), (unsigned char *) (address + addressOffset)) == NETIF_NO_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}
		
  return i_return;
}

int32_t LLECOM_NETWORK_IMPL_useDHCP(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t use, int8_t getResult)
{
	int32_t i_return;
	
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d\n", __func__, (netifName + netifNameOffset), getResult); 
	#endif

	if (NETIF_set_is_static_f((char*) (netifName + netifNameOffset), (use == JTRUE) ? 0 : 1) == NETIF_NO_ERROR)
	{
		i_return = 0;
	}
	else
	{
		i_return = J_EUNKNOWN;
	}
	
  return i_return;
}

int32_t LLECOM_NETWORK_IMPL_useStaticDNS(int8_t* netifName, int32_t netifNameOffset, int32_t netifNameLength, int8_t use, int8_t getResult)
{
	ASSERT_LLECOM_NETWORK(netifName != NULL);
	ASSERT_LLECOM_NETWORK(netifNameOffset < netifNameLength);
	
	#ifdef LLECOM_NETWORK_ENABLE_TRACE
	printf("%s: %s %d %d\n", __func__, (netifName + netifNameOffset), use, getResult);
	#endif

	puts("WARNING: a statically set DNS address will be overriden by DHCP when joining a network");
	
  return 0;
}
