/*
 * C
 *
 * Copyright 2017-2018 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */

/**
 * @file
 * @brief Platform implementation specific macro.
 * @author MicroEJ Developer Team
 * @version 0.2.0
 * @date 21 December 2017
 */

#ifndef  LLNET_CONFIGURATION_H
#define  LLNET_CONFIGURATION_H

#include <async_netconn_sockets.h>

/**
 * @brief Compatibility sanity check value.
 * This define value is checked in the implementation to validate that the version of this configuration
 * is compatible with the implementation.
 *
 * This value must not be changed by the user of the CCO.
 * This value must be incremented by the implementor of the CCO when a configuration define is added, deleted or modified.
 */
#define LLNET_CONFIGURATION_VERSION (1)

/**
 * All the llnet_* functions are mapped on custom functions from the async_netconn component.
 */
#define llnet_accept		async_netconn_accept
#define llnet_bind			async_netconn_bind
#define llnet_close			async_netconn_close
#define llnet_connect		async_netconn_connect
#define llnet_fcntl			async_netconn_fcntl
#define llnet_gethostname	gethostname
#define llnet_getpeername	async_netconn_getpeername
#define llnet_getsockname	async_netconn_getsockname
#define llnet_getsockopt	async_netconn_getsockopt
#define llnet_htonl			lwip_htonl
#define llnet_htons			lwip_htons
#define llnet_ioctl			async_netconn_ioctl
#define llnet_listen		async_netconn_listen
#define llnet_ntohs			lwip_ntohs
#define llnet_recv			async_netconn_recv
#define llnet_recvfrom		async_netconn_recvfrom
#define llnet_send			async_netconn_send
#define llnet_sendto		async_netconn_sendto
#define llnet_setsockopt	async_netconn_setsockopt
#define llnet_socket		async_netconn_socket
#define llnet_shutdown		async_netconn_shutdown
#define llnet_errno         async_netconn_errno

/**
 *  Address family:
 *  - AF_INET for IPv4
 *  - AF_INET6 for IPv6
 */
#define ADDRESS_FAMILY (AF_INET)

/*
 * Initialize the network stack.
 * Returns 0 on success, -1 on error.
 * This method may block and must be called in another RTOS task.
 * See async_netconn task.
 * By default this macro does nothing.
 */
#define llnet_init()

/*
 * Returns true (bool) if we can call the services of the network stack without jeopardizing the system,
 * otherwise returns false (bool).
 * Returning true does not mean that a network interface is up.
 * By default this macro always return true.
 */
#define llnet_is_ready()	(true)

/**
 * Enable network debug trace
 */
//#define LLNET_DEBUG

/**
 * If ioctl() cannot be used to implement the LLNET_STREAMSOCKETCHANNEL_IMPL_available() function,
 * define USE_MSG_PEEK_FOR_AVAILABLE and NET_EMBEDDED_AVAILABLE_BUFFER_SIZE.
 */
#define USE_MSG_PEEK_FOR_AVAILABLE
#define NET_EMBEDDED_AVAILABLE_BUFFER_SIZE (TCP_WND_DEFAULT)

/**
 * Define USE_IOCTL_FOR_BLOCKING_OPTION to use ioctl() instead of fcntl() to configure socket
 * blocking or non-blocking mode.
 */
//#define USE_IOCTL_FOR_BLOCKING_OPTION

/**
 * Define the mapping between the LLNET provided network interface names
 * and the platform dependent network interface names
 * - at platform level, the netif names are created by LwIP
 * - the loopback netif is always the first one created and maintained forever
 * - the station and soft ap netifs can change order, depending on how the application will start/stop them
 */
#if (defined(LWIP_NETIF_LOOPBACK) && (LWIP_NETIF_LOOPBACK != 0))
#define LLNET_NETIF_NAMES    {"loopback_interface", "wifi_interface", "wifi_interface", "wifi_interface", "wifi_interface"}
#define PLATFORM_NETIF_NAMES {"lo0"               , "st1"           , "st2"           , "ap1"           , "ap2"           }
#else
#define LLNET_NETIF_NAMES    {"wifi_interface", "wifi_interface", "wifi_interface", "wifi_interface"}
#define PLATFORM_NETIF_NAMES {"st0"           , "st1"           , "ap0"           , "ap1"           }
#endif

#endif // LLNET_CONFIGURATION_H
