/*
 * C
 *
 * Copyright 2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

#ifndef NET_DELEGATE_TASK_H
#define NET_DELEGATE_TASK_H

#include <stdint.h>

#if !defined(LLNET_DEBUG_TRACE) || !defined(LLNET_DEBUG_ASSERT)
// comment this define to remove network debug trace
//#define LLNET_DEBUG

#ifdef LLNET_DEBUG
#define LLNET_DEBUG_TRACE 	printf
#define LLNET_DEBUG_ASSERT(...) 	printf(__VA_ARGS__); while(1);
#else
#define LLNET_DEBUG_TRACE(...) ((void) 0)
#define LLNET_DEBUG_ASSERT(...) ((void) 0)
#endif
#endif

/** @brief define the amount of message in private mailbox of module */
#define NET_DELEGATE_MAX_NUMBER_OF_EVENT_IN_MAILBOX  (5)

/** @brief list of module constant */
typedef enum
{
	NET_NO_ERROR,
	NET_ADAPTATION_ERROR_INIT,
	NET_MODULE_NOT_INITIALIZED,
	NET_PARAMETER_ERROR,
	NET_UNEXPECTED_ERROR_IN_RECEIVED,
	NET_UNKNOWN_EVENT_RECEIVED,
	NET_ERROR_DURING_FREEING_POOL,
	NET_NO_SPACE_AVAILABLE,
	NET_CANNOT_SUSPEND_JVM_THREAD,
	NET_CANNOT_RESUME_JVM_THREAD,
	NET_CANNOT_GET_JVM_THREAD,
	NET_EVENT_NOT_FOUND,
}NET_status_t;

/** @brief list of event module's */
typedef enum
{
	/* interface management */
	NET_START_EVENT,
	NET_STOP_EVENT,
	NET_APPLY_CONFIGURATION_EVENT,
	
	/* datagram socket channel */
	NET_RECEIVE_EVENT,
	NET_SEND_EVENT,

	/* common socket channel */
	NET_CONNECT_EVENT,
	NET_CLOSE_EVENT,
	NET_SET_BLOCKING_EVENT,
	NET_SHUTDOWN_EVENT,
	NET_BIND_EVENT,
	NET_GET_OPTION_EVENT,
	NET_GET_OPTION_AS_BYTEARRAY_EVENT,
	NET_SET_OPTION_EVENT,
	NET_SET_OPTION_AS_BYTEARRAY_EVENT,
	NET_GET_LOCAL_PORT_EVENT,
	NET_GET_PEER_PORT_EVENT,
	NET_GET_LOCAL_ADDRESS_EVENT,
	NET_GET_PEER_ADDRESS_EVENT,
	NET_SOCKET_EVENT,
	NET_AVAILABLE_EVENT,
	
	/* network address */
	NET_ATON_EVENT,
	NET_GET_LOCAL_HOSTNAME_NATIVE_EVENT,
	NET_LOOKUPINADDRANY_EVENT,
	NET_LOOPBACK_ADDRESS_EVENT,
	
	/* network interface */
	NET_GET_VM_INTERFACE_EVENT,
	NET_GET_VM_INTERFACE_COUNT_EVENT,
	NET_GET_VM_INTERFACE_ADDRESS_EVENT,
	NET_GET_VM_INTERFACE_ADDRESS_COUNT_EVENT,
	NET_IS_LOOPBACK_EVENT,
	NET_IS_POINT_TO_POINT_EVENT,
	NET_IS_UP_EVENT,
	NET_SUPPORT_MULTICAST_EVENT,
	NET_GET_HARDWARE_ADDRESS_EVENT,
	
	/* domain specific */
	NET_RESOLVE_DNS_EVENT,

	/* stream socket channel */
	NET_READ_BYTE_BUFFER_NATIVE_EVENT,
	NET_READ_EVENT,
	NET_WRITE_BYTE_BUFFER_NATIVE_EVENT,
	NET_WRITE_EVENT,
	NET_ACCCEPT_EVENT,
	NET_LISTEN_EVENT,

	/* ssl specific */
	NET_SSL_INITIAL_CLIENT_HANDSHAKE_EVENT,
	NET_SSL_INITIAL_SERVER_HANDSHAKE_EVENT,
	NET_SSL_READ_EVENT,
	NET_SSL_WRITE_EVENT,
	NET_SSL_CLOSE_EVENT,
	NET_SSL_FREE_SSL_EVENT,
	NET_SSL_CREATE_EVENT,
	NET_SSL_AVAILABLE_EVENT,
	NET_SSL_CREATE_CONTEXT_EVENT,
	NET_SSL_PARSE_CERT_EVENT,
	NET_SSL_ADD_TRUSTED_CERT_EVENT,
	NET_SSL_CLEAR_TRUST_STORE_EVENT,
	NET_SSL_CLEAR_KEY_STORE_EVENT,
	NET_SSL_SET_CERTIFICATE_EVENT,
	NET_SSL_SET_PRIVATE_KEY_EVENT,
	NET_SSL_INIT_CHAIN_BUFFER_EVENT,
	NET_SSL_ADD_CHAIN_CERTIFICATE_EVENT,
	NET_SSL_CLOSE_CONTEXT_EVENT,
	
	/* wifi specific */
	NET_WIFI_DISABLE_SOFT_AP_EVENT,
	NET_WIFI_ENABLE_SOFT_AP_EVENT,
	NET_WIFI_SET_NAME_SOFT_AP_EVENT,
	NET_WIFI_GET_BSSID_EVENT,
	NET_WIFI_GET_CHANNEL_EVENT,
	NET_WIFI_GET_RSSI_EVENT,
	NET_WIFI_GET_SECURITY_MODE_EVENT,
	NET_WIFI_GET_SSID_EVENT,
	NET_WIFI_GET_WPS_MODES_EVENT,
	NET_WIFI_JOIN_EVENT,
	NET_WIFI_LEAVE_EVENT,
	NET_WIFI_GET_AP_COUNT_EVENT,
	NET_WIFI_GET_AP_SSID_EVENT,
	NET_WIFI_GET_AP_BSSID_EVENT,
	NET_WIFI_GET_AP_CHANNEL_EVENT,
	NET_WIFI_GET_AP_RSSI_EVENT,
	NET_WIFI_GET_AP_SECURITY_MODE_EVENT,
	NET_WIFI_GET_AP_WPS_MODES_EVENT,
	NET_WIFI_GET_CLIENT_STATE,

	NET_MAX_EVENT,  /* no insertion after this line */
}NET_id_event_t;

/** list of module operations group */
typedef enum
{
	NET_READ,
	NET_WRITE,
	NET_ACCEPT_CONNECT
}NET_id_operation_t;

/** @brief define argument for aton event */
typedef struct
{
	char * pc_aton_in;                  /**< in pointer for function */
	unsigned short us_length_in;        /**< in address index */
	short s_error_out;                  /**< out result */
}NET_aton_ctx_t;

/** @brief define argument for interface address count event */
typedef struct
{
	unsigned long ul_if_index_in;          /**< in interface index */
	char * pc_ifname_in;                   /**< in interface name pointer */
	unsigned long ul_ifname_length_in;     /**< in interface name  length */
	int i_error_out;                       /**< out result */
}NET_if_ctx_t;

/** @brief define argument for interface address count event */
typedef struct
{
	unsigned long ul_if_index_in;          /**< in interface index */
	unsigned long ul_addr_index_in;        /**< in address index */
	char * pc_addr_in;                     /**< in address pointer */
	unsigned long ul_addr_length_in;       /**< in address length */
	int i_error_out;                       /**< out result */
}NET_if_address_ctx_t;

/** @brief define argument for interface address count event */
typedef struct
{
	unsigned long ul_index_in;          /**< in address index */
	short s_error_out;                  /**< out result */
}NET_if_address_count_ctx_t;

/** @brief define argument for get interface hardware addresss */
typedef struct
{
	char * pc_ifname_in;                    /**< in interface name */
	unsigned long ul_ifname_length_in;      /**< in data length pointer by generic pointer set */
	unsigned char * puc_hw_addr_in;         /**< in generic pointer */
	unsigned long ul_hw_length_in;          /**< in data length pointer by generic pointer set */
	short s_error_out;                      /**< out result */
}NET_network_interface_get_hw_option_ctx_t;

/** @brief define argument for generic get option socket event */
typedef struct
{
	char * pc_generic_in;               /**< in generic pointer */
	unsigned long ul_length_in;         /**< in data length pointer by generic pointer set */
	short s_error_out;                  /**< out result */
}NET_generic_network_interface_get_option_ctx_t;

/** @brief define argument for generic network address event */
typedef struct
{
	char * pc_generic_in;               /**< in generic pointer */
	unsigned long ul_length_in;         /**< in data length pointer by generic pointer set */
	short s_error_out;                  /**< out result */
}NET_network_address_ctx_t;

/** @brief define argument for generic get socket port event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	int i_error_out;                    /**< out result */
}NET_generic_get_socket_port_field_ctx_t;

/** @brief define argument for generic get socket address event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	unsigned char * puc_name_in;                  /**< in name */
	unsigned short us_length_in;        /**< in name length */
	short s_error_out;                  /**< out result */
}NET_generic_get_socket_address_field_ctx_t;

/** @brief define argument for socket event */
typedef struct
{
	unsigned char uc_type_in;         /**< in stream */
	short s_error_out;                  /**< out result */
}NET_socket_ctx_t;

/** @brief define argument for available event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	short s_error_out;                  /**< out result */
}NET_available_ctx_t;

/** @brief define argument for set option event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor to receive */
	unsigned long ul_option_in;         /**< in option to set */
	unsigned long ul_value_in;          /**< in option value to set */
	short s_error_out;                  /**< out result */
}NET_set_option_ctx_t;

/** @brief define argument for set option as byte array event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor to receive */
	unsigned long ul_option_in;         /**< in option to set */
	char* pc_value_in;       			/**< in option value pointer to set */
	unsigned long ul_value_length_in;   /**< in option value length to set */
	short s_error_out;                  /**< out result */
}NET_set_option_as_bytearray_ctx_t;

/** @brief define argument for get option event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor to receive */
	unsigned long ul_option_in;         /**< in option to set */
	long l_error_out;                   /**< out result */
}NET_get_option_ctx_t;

/** @brief define argument for get option event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor to receive */
	unsigned long ul_option_in;         /**< in option to set */
	char* pc_dst_out;         			/**< out pointer to set */
	unsigned long ul_dst_length_in;     /**< in pointer length */
	long l_error_out;                   /**< out result */
}NET_get_option_as_bytearray_ctx_t;

/** @brief define argument for set blocking event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor to receive */
	unsigned char uc_blocking_in;       /**< in blocking or not */
	short s_error_out;                  /**< out result */
}NET_set_blocking_ctx_t;

/** @brief define argument for set blocking event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor to send */
	unsigned char * puc_buffer_in;       /**< in address to bind socket */
	unsigned short us_length_buff_in;   /**< in length address buffer */
	unsigned short us_port_in;          /**< in port number */
	short s_error_out;                  /**< out result */
}NET_bind_ctx_t;

/** @brief define argument for receive event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor to receive */
	char * pc_buffer_in;                /**< in address of out buffer */
	unsigned short us_length_buff_in;   /**< in length out buffer */
	unsigned char * puc_host_out;				/**< out remote host (adress, port) buffer */
	unsigned short us_host_length_in;		/**< in remote host buffer length */
	long long ll_error_out;             /**< out result */
}NET_receive_ctx_t;

/** @brief define argument for send event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor to send */
	char * pc_buffer_in;                /**< in address of src buffer */
	unsigned short us_length_buff_in;   /**< in length src buffer */
	unsigned char * puc_addr_in;        /**< in address as array of byte */
	unsigned short us_length_addr_in;   /**< in length address */
	unsigned short us_port_in;          /**< in port number */
	short s_error_out;                  /**< out result */
}NET_send_ctx_t;

/** @brief define argument for connect event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	unsigned char * puc_addr_in;        /**< in addr */
	unsigned short us_length_addr_in;   /**< in length addr */
	unsigned short us_port_in;          /**< in port */
	unsigned short us_timeout_in;       /**< in timeout */
	short s_error_out;                  /**< out result */
}NET_connect_ctx_t;

/** @brief define argument for read byte buffer event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	unsigned short us_kind_in;          /**< in kind */
	unsigned char * puc_buffer_out;     /**< out pointer on out buffer */
	unsigned short us_buffer_length_out;/**< out buffer length */
	short s_error_out;                  /**< out result or port */
}NET_read_byte_buffer_ctx_t;

/** @brief define argument for read event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	unsigned short us_kind_in;          /**< in kind */
	short s_error_out;                  /**< out result or port */
}NET_read_ctx_t;

/** @brief define argument for write byte buffer event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	unsigned short us_kind_in;          /**< in kind */
	unsigned char * puc_buffer_in;      /**< in pointer on in buffer */
	unsigned short us_buffer_length_in; /**< in buffer length */
	short s_error_out;                  /**< out result or port */
}NET_write_byte_buffer_ctx_t;

/** @brief define argument for write event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	unsigned short us_kind_in;          /**< in kind */
	short s_error_out;                  /**< out result or port */
	char c_towrite_in;                  /**< in byte to write */
}NET_write_ctx_t;

/** @brief define argument for accept event*/
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	short s_error_out;                  /**< out result or port */
}NET_accept_ctx_t;

/** @brief define argument for get gps position event */
typedef struct
{
	char * pc_buffer_in;                /**< in address of out buffer */
	unsigned short us_length_buff_in;   /**< in length out buffer */
	short s_error_out;                  /**< out result */
}NET_get_gps_position_ctx_t;

/** @brief define argument for SSL/TLS initial client handshake event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	int i_ssl_socket_id_in;   			/**< in ssl socket id */
	short s_error_out;                  /**< out result */
}NET_SSL_initial_client_handshake_ctx_t;

/** @brief define argument for SSL/TLS initial server handshake event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	int i_ssl_socket_id_in;   			/**< in ssl socket id */
	short s_error_out;                  /**< out result */
}NET_SSL_initial_server_handshake_ctx_t;

/** @brief define argument for SSL/TLS read event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	int i_ssl_socket_id_in;   			/**< in ssl socket id */
	unsigned char * puc_buffer_out;     /**< out pointer on out buffer */
	unsigned int ui_buffer_length_out;	/**< out buffer length */
	int i_error_out;                    /**< out result */
}NET_SSL_read_ctx_t;

/** @brief define argument for SSL/TLS write event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	int i_ssl_socket_id_in;   			/**< in ssl socket id */
	unsigned char * puc_buffer_in;      /**< in pointer on in buffer */
	unsigned int ui_buffer_length_in; 	/**< in buffer length */
	int i_error_out;                    /**< out result or port */
}NET_SSL_write_ctx_t;

/** @brief define argument for SSL/TLS close event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	int i_ssl_socket_id_in;   			/**< in ssl socket id */
	unsigned char uc_autoclose_in;      /**< in autoclose socket */
	short s_error_out;                  /**< out result or port */
}NET_SSL_close_ctx_t;

/** @brief define argument for SSL/TLS free SSL context event */
typedef struct
{
	int i_ssl_socket_id_in;   			/**< in ssl socket id */
	short s_error_out;                  /**< out result or port */
}NET_SSL_free_ssl_ctx_t;

/** @brief define argument for SSL/TLS create event */
typedef struct
{
	int i_context_in;			        /**< in context */
	unsigned short us_fd_in;            /**< in file descriptor */
	unsigned char uc_autoclose_in;      /**< in autoclose socket */
	unsigned char uc_use_client_mode_in;  /**< in use client mode */
	unsigned char uc_need_client_auth_in; /**< in need client auth */
	int i_error_out;                  /**< out result or port */
}NET_SSL_create_ctx_t;

/** @brief define argument for SSL/TLS available event */
typedef struct
{
	int i_ssl_socket_id_in;			  /**< in ssl socket id */
	int i_error_out;                  /**< out result or port */
}NET_SSL_available_ctx_t;

/** @brief define argument for SSL/TLS parse certificat event */
typedef struct
{
	int i_cert_in;					  /**< in certificate buffer pointer */
	int i_off_in;					  /**< in certificate buffer offset */
	int i_len_in;					  /**< in certificate buffer length */
	int i_error_out;                  /**< out result or port */
}NET_SSL_parse_cert_ctx_t;

/** @brief define argument for SSL/TLS create client context event */
typedef struct
{
	int i_protocol_in;					  /**< in protocol */
	unsigned char uc_is_client_context_in;  /**< in is client context */
	int i_error_out;                  	  /**< out result or port */
}NET_SSL_create_context_ctx_t;

/** @brief define argument for SSL/TLS add trusted certificate event */
typedef struct
{
	int i_context_in;			        /**< in context */
	int i_cert_in;					  /**< in certificate buffer pointer */
	int i_off_in;					  /**< in certificate buffer offset */
	int i_len_in;					  /**< in certificate buffer length */
	int i_format_in;					  /**< in certificate format */
	int i_error_out;                  	  /**< out result or port */
}NET_SSL_add_trusted_cert_ctx_t;

/** @brief define argument for SSL/TLS set certificate event */
typedef struct
{
	int i_context_in;			        /**< in context */
	int i_cert_in;					  /**< in certificate buffer pointer */
	int i_off_in;					  /**< in certificate buffer offset */
	int i_len_in;					  /**< in certificate buffer length */
	int i_format_in;					  /**< in certificate format */
	int i_error_out;                  	  /**< out result or port */
}NET_SSL_set_cert_ctx_t;

/** @brief define argument for SSL/TLS set private key event */
typedef struct
{
	int i_context_in;			        /**< in context */
	int i_private_key_in;				/**< in private key buffer pointer */
	int i_off_in;					  /**< in private key buffer offset */
	int i_len_in;					  /**< in private key buffer length */
	int i_password_in;					  /**< in password buffer pointer */
	int i_password_off_in;					  /**< in password buffer offset */
	int i_password_len_in;					  /**< in password buffer length */
	int i_error_out;                  	  /**< out result or port */
}NET_SSL_set_private_key_ctx_t;

/** @brief define argument for SSL/TLS clear trust store event */
typedef struct
{
	int i_context_in;			        /**< in context */
	int i_error_out;                  	  /**< out result or port */
}NET_SSL_clear_trust_store_ctx_t;

/** @brief define argument for SSL/TLS clear key store event */
typedef struct
{
	int i_context_in;			        /**< in context */
	int i_error_out;                  	  /**< out result or port */
}NET_SSL_clear_key_store_ctx_t;

/** @brief define argument for SSL/TLS init chain buffer event */
typedef struct
{
	int i_context_in;			        /**< in context */
	int i_nb_chain_certs_in;            /**< in number of certificates in the chain */
	int i_chain_certs_total_size_in;    /**< in the total size of certificates in the chain */
	int i_error_out;                  	  /**< out result or port */
}NET_SSL_init_chain_buffer_ctx_t;

/** @brief define argument for SSL/TLS add chain certificate event */
typedef struct
{
	int i_context_in;			        /**< in context */
	int i_cert_in;					  /**< in certificate buffer pointer */
	int i_off_in;					  /**< in certificate buffer offset */
	int i_len_in;					  /**< in certificate buffer length */
	int i_format_in;					  /**< in certificate format */
	int i_chain_buffer_size_in;            /**< in chain buffer size */
	int i_error_out;                  	  /**< out result or port */
}NET_SSL_add_chain_cert_ctx_t;

/** @brief define argument for SSL/TLS close context event */
typedef struct
{
	int i_context_in;			        /**< in context */
	int i_error_out;                  	/**< out result or port */
}NET_SSL_close_context_ctx_t;

/** @brief define argument for start event */
typedef struct
{
	int i_interface_name_length_in;			/**< in interface name length */
	char * pc_interface_name_in;				/**< in interface name */
	short s_error_out;                  /**< out result */
}NET_start_ctx_t;

/** @brief define argument for stop event */
typedef struct
{
	int i_interface_name_length_in;			/**< in interface name length */
	char * pc_interface_name_in;         /**< in interface name */
	short s_error_out;                  /**< out result */
}NET_stop_ctx_t;

/** @brief define argument for apply configuration event */
typedef struct
{
	int i_interface_name_length_in;			/**< in interface name length */
	char * pc_interface_name_in;				/**< in interface name */
	short s_error_out;                  /**< out result */
}NET_apply_configuration_ctx_t;

/** @brief define argument for disable wifi ap event */
typedef struct
{
	short s_error_out;                  /**< out result */
}NET_wifi_disable_soft_ap_ctx_t;

/** @brief define argument for enable wifi ap event */
typedef struct
{
	char * pc_ssid_in;                  /**< in wifi ssid */
	char * pc_passphrase_in;           	/**< in passphrase ssid */
	int i_ssid_length_in;								/**< in ssid buffer length */
	int i_passphrase_length_in;					/**< in passphrase buffer length */
	short s_error_out;                  /**< out result */
}NET_wifi_enable_soft_ap_ctx_t;

/** @brief define argument for set name wifi ap event */
typedef struct
{
	char * pc_name_in;                  /**< in device name */
	int i_name_length_in;								/**< in device name length */
	short s_error_out;                  /**< out result */
}NET_wifi_set_name_soft_ap_ctx_t;

/** @brief define argument for get bssid wifi event */
typedef struct
{
	char * pc_bssid_in;									/**< in wifi bssid */
	int i_bssid_length_in;							/**< in bssid buffer length */
	short s_error_out;                  /**< out result */
}NET_wifi_get_bssid_ctx_t;

/** @brief define argument for generic event with only return */
typedef struct
{
	short s_error_out;                  /**< out result */
}NET_generic_only_return_ctx_t;

/** @brief define argument for get rssi wifi event */
typedef struct
{
	float * pf_rssi_in;									/**< in wifi rssi */
	int i_rssi_length_in;								/**< in rssi buffer length */
	short s_error_out;                  /**< out result */
}NET_wifi_get_rssi_ctx_t;


/** @brief define argument for get ssid wifi event */
typedef struct
{
	char * pc_ssid_in;                  /**< in wifi ssid */
	int i_ssid_length_in;								/**< in ssid buffer length */
	short s_error_out;                  /**< out result */
}NET_wifi_get_ssid_ctx_t;


/** @brief define argument for join wifi event */
typedef struct
{
	char * pc_ssid_in;                  /**< in wifi ssid */
	char * pc_passphrase_in;           	/**< in passphrase ssid */
	int i_ssid_length_in;								/**< in ssid buffer length */
	int i_passphrase_length_in;					/**< in passphrase buffer length */
	int i_security_mode_in;						/**< in security mode */
	short s_error_out;                  /**< out result */
}NET_wifi_join_ctx_t;

/** @brief define argument for get ap ssid wifi event */
typedef struct
{
	int i_index_in;											/**< in index */
	unsigned char * puc_ssid_in;				/**< in wifi ssid */
	int i_ssid_length_in;								/**< in ssid buffer length */
	short s_error_out;                  /**< out result */
}NET_wifi_get_ap_ssid_ctx_t;

/** @brief define argument for get ap bssid wifi event */
typedef struct
{
	int i_index_in;											/**< in index */
	unsigned char * puc_bssid_in;				/**< in wifi bssid */
	int i_bssid_length_in;							/**< in bssid buffer length */
	short s_error_out;                  /**< out result */
}NET_wifi_get_ap_bssid_ctx_t;

/** @brief define argument for get ap channel wifi event */
typedef struct
{
	int i_index_in;											/**< in index */
	short s_error_out;                  /**< out result */
}NET_wifi_get_ap_channel_ctx_t;

/** @brief define argument for get ap rssi wifi event */
typedef struct
{
	int i_index_in;											/**< in index */
	float * pf_rssi_in;									/**< in wifi rssi */
	int i_rssi_length_in;								/**< in rssi buffer length */
	short s_error_out;                  /**< out result */
}NET_wifi_get_ap_rssi_ctx_t;

/** @brief define argument for get ap security mode wifi event */
typedef struct
{
	int i_index_in;											/**< in index */
	short s_error_out;                  /**< out result */
}NET_wifi_get_ap_security_mode_ctx_t;

/** @brief define argument for get ap wps modes wifi event */
typedef struct
{
	int i_index_in;											/**< in index */
	short s_error_out;                  /**< out result */
}NET_wifi_get_ap_wps_modes_ctx_t;

/** @brief define argument for close event */
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	short s_error_out;                  /**< out result or port */
}NET_close_ctx_t;

/** @brief define argument for resolve domain event */
typedef struct
{
	char * pc_host_in_out;       	      /**< in host to resolve, out address retrieve */
	unsigned short us_length_in;        /**< in buffer length */
	short s_error_out;                  /**< out result or port */
}NET_resolve_domain_ctx_t;

/** @brief define argument for listen event*/
typedef struct
{
	unsigned short us_fd_in;            /**< in file descriptor */
	short s_error_out;                  /**< out result or port */
}NET_listen_ctx_t;

/** @brief define argument for get client state*/
typedef struct
{
	short s_error_out;                  /**< out result */
}NET_get_client_state_ctx_t;

/** @brief union with all parameter available */
typedef union{

	NET_receive_ctx_t st_receive_ctx_t;
	NET_send_ctx_t st_send_ctx_t;
	NET_connect_ctx_t st_connect_ctx_t;
	NET_close_ctx_t st_close_ctx_t;
	NET_resolve_domain_ctx_t st_resolve_domain_ctx_t;
	NET_read_byte_buffer_ctx_t st_read_byte_buffer_ctx_t;
	NET_read_ctx_t st_read_ctx_t;
	NET_write_byte_buffer_ctx_t st_write_byte_buffer_ctx_t;
	NET_write_ctx_t st_write_ctx_t;
	NET_accept_ctx_t st_accept_ctx_t;
	NET_listen_ctx_t st_listen_ctx_t;
	NET_get_gps_position_ctx_t st_get_gps_position_ctx_t;
	NET_SSL_initial_client_handshake_ctx_t st_ssl_initial_client_handshake_ctx_t;
	NET_SSL_initial_server_handshake_ctx_t st_ssl_initial_server_handshake_ctx_t;
	NET_SSL_read_ctx_t st_ssl_read_ctx_t;
	NET_SSL_write_ctx_t st_ssl_write_ctx_t;
	NET_SSL_close_ctx_t st_ssl_close_ctx_t;
	NET_SSL_free_ssl_ctx_t st_ssl_free_ssl_ctx_t;
	NET_SSL_create_ctx_t st_ssl_create_ctx_t;
	NET_SSL_available_ctx_t st_ssl_available_ctx_t;
	NET_SSL_parse_cert_ctx_t st_ssl_parse_cert_ctx_t;
	NET_SSL_create_context_ctx_t st_ssl_create_context_ctx_t;
	NET_SSL_add_trusted_cert_ctx_t st_ssl_add_trusted_cert_ctx_t;
	NET_SSL_set_cert_ctx_t st_ssl_set_cert_ctx_t;
	NET_SSL_set_private_key_ctx_t st_ssl_set_private_key_ctx_t;
	NET_SSL_clear_trust_store_ctx_t st_ssl_clear_trust_store_ctx_t;
	NET_SSL_clear_key_store_ctx_t st_ssl_clear_key_store_ctx_t;
	NET_SSL_init_chain_buffer_ctx_t st_ssl_init_chain_buffer_ctx_t;
	NET_SSL_add_chain_cert_ctx_t st_ssl_add_chain_cert_ctx_t;
	NET_SSL_close_context_ctx_t st_ssl_close_context_ctx_t;
	NET_start_ctx_t st_start_ctx_t;
	NET_stop_ctx_t st_stop_ctx_t;
	NET_apply_configuration_ctx_t st_apply_configuration_ctx_t;
	NET_wifi_disable_soft_ap_ctx_t st_wifi_disable_soft_ap_ctx_t;
	NET_wifi_enable_soft_ap_ctx_t st_wifi_enable_soft_ap_ctx_t;
	NET_wifi_set_name_soft_ap_ctx_t st_wifi_set_name_soft_ap_ctx_t;
	NET_wifi_join_ctx_t st_wifi_join_ctx_t;
	NET_generic_only_return_ctx_t st_generic_only_return_ctx_t;
	NET_wifi_get_bssid_ctx_t st_wifi_get_bssid_ctx_t;
	NET_wifi_get_rssi_ctx_t st_wifi_get_rssi_ctx_t;
	NET_wifi_get_ssid_ctx_t st_wifi_get_ssid_ctx_t;
	NET_wifi_get_ap_ssid_ctx_t st_wifi_get_ap_ssid_ctx_t;
	NET_wifi_get_ap_bssid_ctx_t st_wifi_get_ap_bssid_ctx_t;
	NET_wifi_get_ap_channel_ctx_t st_wifi_get_ap_channel_ctx_t;
	NET_wifi_get_ap_rssi_ctx_t st_wifi_get_ap_rssi_ctx_t;
	NET_wifi_get_ap_security_mode_ctx_t st_wifi_get_ap_security_mode_ctx_t;
	NET_wifi_get_ap_wps_modes_ctx_t st_wifi_get_ap_wps_modes_ctx_t;
	NET_set_option_ctx_t st_set_option_ctx_t;
	NET_set_option_as_bytearray_ctx_t st_set_option_as_bytearray_ctx_t;
	NET_get_option_ctx_t st_get_option_ctx_t;
	NET_get_option_as_bytearray_ctx_t st_get_option_as_bytearray_ctx_t;
	NET_set_blocking_ctx_t st_set_blocking_ctx_t;
	NET_bind_ctx_t st_bind_ctx_t;
	NET_network_address_ctx_t st_network_address_ctx_t;
	NET_available_ctx_t st_available_ctx_t;
	NET_socket_ctx_t st_socket_ctx_t;
	NET_generic_get_socket_address_field_ctx_t st_generic_get_socket_address_field_ctx_t;
	NET_generic_get_socket_port_field_ctx_t st_generic_get_socket_port_field_ctx_t;
	NET_generic_network_interface_get_option_ctx_t st_generic_network_interface_get_option_ctx_t;
	NET_network_interface_get_hw_option_ctx_t st_network_interface_get_hw_option_ctx_t;
	NET_if_address_count_ctx_t st_if_address_count_ctx_t;
	NET_if_address_ctx_t st_if_address_ctx_t;
	NET_if_ctx_t st_if_ctx_t;
	NET_aton_ctx_t st_aton_ctx_t;
	NET_get_client_state_ctx_t st_get_client_state_ctx_t;
}NET_generic_ctx_t;

/** @brief define an event */
typedef struct
{
	NET_id_event_t e_id_event;               /**< id event */
	NET_generic_ctx_t u_ctx_event;           /**< event context */
	unsigned long ul_id_java_thread;           /**< id original java thread */
}NET_event_t;


/**
 * @brief function to initialise the delegate module
 *
 * @return @see NET_status_t
 */
NET_status_t NET_init_f(void);

/**
 * @brief function to get an event of private delegate module pool
 *
 * @param[out] _pst_event pointer of pointer of event
 *
 * @return @see NET_status_t
 */
NET_status_t NET_get_event(NET_event_t ** _pst_event);

/**
 * @brief function to release an event of private delegate module pool
 *
 * @param[in] _pst_event pointer of event to release
 *
 * @return @see NET_status_t
 */
NET_status_t NET_release_event(NET_event_t * _pst_event);

/**
 * @brief function to post event on delegate module
 *
 * This function suspend the JVM thread caller if no error occurred.
 *
 * @param[in,out] _pst_event pointer of event
 *
 * @return @see NET_status_t
 */
NET_status_t NET_post_event(NET_event_t * _pst_event);

/**
 * @brief function to repost event on delegate module
 *
 * @param[in,out] _pst_event pointer of event
 *
 * @return @see NET_status_t
 */
NET_status_t NET_repost_event(NET_event_t * _pst_event);

/**
 * @brief function to retrieve the event on delegate module, corresponding with java thread id
 *
 * @param[out] _pst_event pointer of pointer of event
 * @param[in] _ul_id_java_thread java thread id 
 *
 * @return @see NET_status_t
 */
NET_status_t NET_retrieve_event(NET_event_t ** _pst_event, unsigned long _ul_id_java_thread);


/**
 * @brief main function of delegate module
 *
 * @todo the function shall notice that error occurred but how ? (log, callback, api status function ...)
 */
void NET_run_f(void);

/**
 * @brief this function notify the delegate when a socket is ready for operation.
 *
 * @param[in] _us_socket_id socket id
 * @param[in] operation operation id on which the socket is ready
 *
 * @return @see NET_status_t
 */
NET_status_t NET_notify_ready_for_operation(unsigned short _us_socket_id, NET_id_operation_t operation);

#endif /* NET_DELEGATE_TASK_H */
