/*
 * C
 *
 * Copyright 2017-2018 IS2T. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break IS2T warranties on the whole library.
 */

#ifndef  ASYNC_NETCONN_CONFIGURATION_H
#define  ASYNC_NETCONN_CONFIGURATION_H

/**
 * @file
 * @brief Asynchronous network based on netconn configuration.
 * @author MicroEJ Developer Team
 * @version 0.0.1
 * @date 22 May 2018
 */

#include <stdint.h>
#include <sni.h>

#ifdef __cplusplus
	extern "C" {
#endif

/**
 * @brief Compatibility sanity check value.
 * This define value is checked in the implementation to validate that the version of this configuration
 * is compatible with the implementation.
 *
 * This value must not be changed by the user of the CCO.
 * This value must be incremented by the implementor of the CCO when a configuration define is added, deleted or modified.
 */
#define ASYNC_NETCONN_CONFIGURATION_VERSION (0)

/**
 * @brief Enable debug logger
 */
//#define ENABLE_ASYNC_NETCONN_DEBUG

/**
 * @brief Enable assert
 */
//#define ENABLE_ASYNC_NETCONN_ASSERT

/**
 * @brief Maximum number of asynchronous netconn requests that can be done at the same moment.
 */
#define MAX_NB_ASYNC_NETCONN (16)

/**
 * @brief async_netconn task stack size in bytes.
 */
#define ASYNC_NETCONN_TASK_STACK_SIZE (2048)

/**
 * @brief async_netconn task name.
 */
#define ASYNC_NETCONN_TASK_NAME	"AsyncNetconn"

/**
 * @brief async_netconn task priority.
 */
#define ASYNC_NETCONN_TASK_PRIORITY	(12)

/**
 * @brief async_netconn mutex name.
 */
#define ASYNC_NETCONN_MUTEX_NAME	"AsyncNetconnMutex"

#ifdef __cplusplus
	}
#endif

#endif // ASYNC_NETCONN_CONFIGURATION_H
