/*
 * C
 *
 * Copyright 2014-2018 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * @file
 * @brief LLNET_NETWORKADDRESS 2.1.0 implementation over BSD-like API.
 * @author MicroEJ Developer Team
 * @version 0.2.0
 * @date 21 December 2017
 */

#include <LLNET_NETWORKADDRESS_impl.h>

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "LLNET_CONSTANTS.h"
#include "LLNET_ERRORS.h"
#include "LLNET_Common.h"

#ifdef __cplusplus
	extern "C" {
#endif

#define LOCALHOST_NAME ("localhost")

int32_t LLNET_NETWORKADDRESS_IMPL_getLocalHostnameNative(int8_t* result, int32_t length, uint8_t retry)
{
	LLNET_DEBUG_TRACE("%s\n", __func__);
	jint ret;
	ret = llnet_gethostname(result, length);
	if(ret == 0){
		// From gethostname() man:
		// * If the null-terminated hostname is too large to fit, then the name is truncated, and no  error
		// * is  returned  (but  see  NOTES  below).  POSIX.1-2001 says that if such
		// * truncation occurs, then it is unspecified whether the  returned  buffer
		// * includes a terminating null byte.
		//
		// To avoid any issue, we set to 0 the last char.
		result[length-1] = 0;
	}else{
		strncpy((char *)result, LOCALHOST_NAME, length);

		// From strncpy() man:
		// * Warning: If there is no null byte among the first n bytes
	    // * of src, the string placed in dest will not be null terminated.
		//
		// To avoid any issue, we set to 0 the last char.
		result[length-1] = 0;
	}
	return 0;
}

int32_t LLNET_NETWORKADDRESS_IMPL_lookupInaddrAny(int8_t* result, int32_t length, uint8_t retry)
{
	LLNET_DEBUG_TRACE("%s\n", __func__);
#if ADDRESS_FAMILY == AF_INET
	if(length < sizeof(struct in_addr)){
		return J_EINVAL;
	}
	*((in_addr_t*)result) = llnet_htonl(INADDR_ANY);

	return sizeof(in_addr_t);

#else  // ADDRESS_FAMILY == AF_INET6
	if(length < sizeof(struct in6_addr)){
		return J_EINVAL;
	}
	*((struct in6_addr*)result) = in6addr_any;

	return sizeof(struct in6_addr);
#endif // ADDRESS_FAMILY == AF_INET
}

int32_t LLNET_NETWORKADDRESS_IMPL_loopbackAddress(int8_t* result, int32_t length, uint8_t retry){
	LLNET_DEBUG_TRACE("%s\n", __func__);
#if ADDRESS_FAMILY == AF_INET

	if(length < sizeof(struct in_addr)){
		return J_EINVAL;
	}
	*((in_addr_t*)result) = llnet_htonl(INADDR_LOOPBACK);

	return sizeof(in_addr_t);

#else  // ADDRESS_FAMILY == AF_INET6
	if(length < sizeof(struct in6_addr)){
		return J_EINVAL;
	}
	*((struct in6_addr*)result) = in6addr_loopback;

	return sizeof(struct in6_addr);
#endif // ADDRESS_FAMILY == AF_INET
}

#ifdef __cplusplus
	}
#endif
